/*
 *  Copyright (c) 2009, Eric Berry <elberry@gmail.com>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *
 *  	* Redistributions of source code must retain the above copyright notice, this
 *  	  list of conditions and the following disclaimer.
 *  	* Redistributions in binary form must reproduce the above copyright notice,
 *  	  this list of conditions and the following disclaimer in the documentation
 *  	  and/or other materials provided with the distribution.
 *  	* Neither the name of the Organization (TellurianRing.com) nor the names of
 *  	  its contributors may be used to endorse or promote products derived from
 *  	  this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package plugindeveloper.ui
import javax.swing.JPanel
import plugindeveloper.PluginConfiguration
import groovy.swing.SwingBuilder
import java.awt.GridBagConstraints as GBC
import javax.swing.SwingConstants as SC

/**
 *
 * @author eberry
 */
class PluginInformationPanel extends UIPanel {

    def PluginConfiguration config
    
    PluginInformationPanel(String propsPath) {
        config = PluginConfiguration.load(propsPath)
    }

    def create() {
        // get the label text from jEdit properties.
        def messages = [
            name: label("name", "Name:"),
            version: label("version", "Version:"),
            autor: label("author", "Author:"),
            storage: label("storage", "Storage:"),
            activation: label("activation", "Activation:"),
            startup_props: label("startup.props", "Startup Properties:"),
            summary: label("summary", "Summary:"),
            description: label("description", "Description:")
        ]

        // create the activation combo box items.
        def activationItems = [
            comboBoxItem("activation.defer", "The first time plugin is used.", PluginConfiguration.Activation.defer),
            comboBoxItem("activation.startup", "When jEdit starts up.", PluginConfiguration.Activation.startup),
            comboBoxItem("activation.properties", "When the following properties are set.", PluginConfiguration.Activation.properties)
        ]

        // get the help message for the storage checkbox.
        def storage_message = label("storage.message", "<html>This plugin saves user settings, and other data to disk.</html>")

        // build the UI.
        def builder = new SwingBuilder()
        def gbc = builder.gbc(gridx: 0, gridy: 0, weightx: 1.0f, weighty: 1.0f, gridwidth: 1, gridheight: 1, fill: GBC.BOTH)
        def panel = builder.panel() {
            gridBagLayout()
            // set up the labels.
            gbc.insets = [10, 10, 0, 0]
            label(text: messages.name, horizontalAlignment: SC.RIGHT, constraints: gbc)
            gbc.insets = [5, 10, 0, 0]
            gbc.gridy++
            label(text: messages.version, horizontalAlignment: SC.RIGHT, constraints: gbc)
            gbc.gridy++
            label(text: messages.author, horizontalAlignment: SC.RIGHT, constraints: gbc)
            gbc.gridy++
            label(text: messages.storage, horizontalAlignment: SC.RIGHT, constraints: gbc)
            gbc.gridy++
            label(text: messages.activation, horizontalAlignment: SC.RIGHT, constraints: gbc)
            gbc.gridy++
            label(text: messages.startup_props, horizontalAlignment: SC.RIGHT, verticalAlignment: SC.TOP, constraints: gbc)
            gbc.gridy++
            label(text: messages.summary, horizontalAlignment: SC.RIGHT, verticalAlignment: SC.TOP, constraints: gbc)
            gbc.gridy++
            label(text: messages.description, horizontalAlignment: SC.RIGHT, verticalAlignment: SC.TOP, constraints: gbc)

            // set up fields
            gbc.gridx = 1
            gbc.gridy = 0
            gbc.weightx = 1.0f
            gbc.insets = [10, 5, 0, 10]
            textField(id: "name_field", text: config.pluginName, constraints: gbc)
            bind(source: name_field, sourceProperty: 'text', target: config, targetProperty: 'pluginName')
            gbc.insets = [5, 5, 0, 10]
            gbc.gridy++
            textField(id: "version_field", text: config.version, constraints: gbc)
            bind(source: version_field, sourceProperty: 'text', target: config, targetProperty: 'version')
            gbc.gridy++
            textField(id: "author_field", text: config.author, constraints: gbc)
            gbc.gridy++
            checkBox(id: "storage_field", text: storage_message, horizontalTextPosition: SC.RIGHT, selected: config.usePluginHome, constraints: gbc)
            bind(source: storage_field, sourceProperty: 'selected', target: config, targetProperty: 'usePluginHome')
            gbc.gridy++
            comboBox(id: "activation_field", items: activationItems, constraints: gbc, itemStateChanged: {
                // if the user selects to have their plugin loaded when some properties are set.
                def useProperties = activation_field?.selectedItem?.value == PluginConfiguration.Activation.properties
                // then we make the start_props_field editable.
                start_props_field.editable = useProperties
            })
            gbc.gridy++
            scrollPane(constraints: gbc) {
                textArea(id: "start_props_field", text: "", columns: 40, rows: 4, lineWrap: true, wrapStyleWord: true, editable: false)
            }

            println("activation: ${config.activation}")
            // If the config.activation is not startup, or defer, then it must be properties.
            if(config.activation != PluginConfiguration.Activation.startup.toString() && config.activation != PluginConfiguration.Activation.defer.toString()) {
                // set the start_props_field to be editable
                start_props_field.editable = true
                // set the text field's content to the activate property value
                start_props_field.text = config.activation
                // and set the selected item to properties in the combo box.
                activation_field.selectedItem = activationItems.find({it.value == PluginConfiguration.Activation.properties.toString()})
            } else {
                // otherwise set the text area to non-editable, and set the combo box to the right item.
                start_props_field.editable = false
                activation_field.selectedItem = activationItems.find({it.value == PluginConfiguration.Activation.valueOf(config.activation)})
            }
            gbc.gridy++
            scrollPane(constraints: gbc) {
                textArea(id: "summary_field", text: config.summary, columns: 40, rows: 4, lineWrap: true, wrapStyleWord: true, editable: true)
            }
            bind(source: summary_field, sourceProperty: 'text', target: config, targetProperty: 'summary')
            gbc.gridy++
            gbc.weighty = 1.0f
            gbc.insets = [5, 5, 10, 10]
            scrollPane(constraints: gbc) {
                textArea(id: "description_field", text: config.description, columns: 40, rows: 8, lineWrap: true, wrapStyleWord: true, editable: true)
            }
            bind(source: description_field, sourceProperty: 'text', target: config, targetProperty: 'description')
        }
        
        return panel;
    }
}

